@@ -157,7 +157,7 @@ cifs_statfs(struct super_block *sb, struct statfs *buf)
 					   able to support more than this, but best to be safe
 					   since Win2k and others can not handle very long filenames */
 	buf->f_files = 0;	/* undefined */
-	buf->f_ffree = -1;	/* unlimited */
+	buf->f_ffree = 0;	/* unlimited */
 
 	rc = CIFSSMBQFSInfo(xid, pTcon, buf, cifs_sb->local_nls);
 
@@ -299,6 +299,7 @@ struct inode_operations cifs_dir_inode_ops = {
 	.revalidate = cifs_revalidate,  
 	.setattr = cifs_setattr,
 	.symlink = cifs_symlink,
+	.mknod   = cifs_mknod,
 };
 
 struct inode_operations cifs_file_inode_ops = {
@@ -441,7 +442,7 @@ static int cifs_oplock_thread(void * dummyarg)
 	int rc;
 
 	daemonize();
-
+	sprintf(current->comm,"cifsoplockd");
 	oplockThread = current;
 	do {
 		set_current_state(TASK_INTERRUPTIBLE);
@@ -463,7 +464,14 @@ static int cifs_oplock_thread(void * dummyarg)
 					CIFS_I(pfile->f_dentry->d_inode)->write_behind_rc 
 						= rc;
 				cFYI(1,("Oplock flush file %p rc %d",pfile,rc));
-				/* send oplock break */
+				if(pfile->private_data) {
+					rc = CIFSSMBLock(0, pTcon, 
+						((struct cifsFileInfo *) pfile->private_data)->netfid,
+						0 /* len */ , 0 /* offset */, 0, 
+						0, LOCKING_ANDX_OPLOCK_RELEASE,
+						0 /* wait flag */);
+					cFYI(1,("Oplock release rc = %d ",rc));
+				}
 				write_lock(&GlobalMid_Lock);
 			} else
 				break;
